@@ -1242,7 +1242,6 @@ substitute_in_string (const GnomeDesktopItem *item,
 		i++;
 
 		switch (arg[i]) {
-		/* FIXME: subst also %U, %F, %N, %D */
 		case '%':
 			g_string_append_c (str, '%');
 			break;
@@ -1348,19 +1347,19 @@ append_app_arg (GSList *vector_list,
 	
 	/* First we subst for some where it could be more then
 	 * one argument */
-	} else if (strcmp (arg, "%F")) {
+	} else if (!strcmp (arg, "%F")) {
 		*added_status = ADDED_ALL;
 		return append_files (vector_list,
 				     args);
-	} else if (strcmp (arg, "%U")) {
+	} else if (!strcmp (arg, "%U")) {
 		*added_status = ADDED_ALL;
 		return append_uris (vector_list,
 				    args);
-	} else if (strcmp (arg, "%D")) {
+	} else if (!strcmp (arg, "%D")) {
 		*added_status = ADDED_ALL;
 		return append_dirs (vector_list,
 				    args);
-	} else if (strcmp (arg, "%N")) {
+	} else if (!strcmp (arg, "%N")) {
 		*added_status = ADDED_ALL;
 		return append_names (vector_list,
 				     args);
@@ -1402,9 +1401,6 @@ ditem_execute (const GnomeDesktopItem *item,
 		const char *options =
 			gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_TERMINAL_OPTIONS);
 
-		term_argc = 0;
-		term_argv = NULL;
-
 		if (options != NULL) {
 			g_shell_parse_argv (options,
 					    &term_argc,
@@ -2172,10 +2168,10 @@ gnome_desktop_item_set_location_file (GnomeDesktopItem *item, const char *file)
 	if (file != NULL) {
 		char *uri;
 		uri = gnome_vfs_get_uri_from_local_path (file);
-		gnome_desktop_item_set_location_file (item, uri);
+		gnome_desktop_item_set_location (item, uri);
 		g_free (uri);
 	} else {
-		gnome_desktop_item_set_location_file (item, NULL);
+		gnome_desktop_item_set_location (item, NULL);
 	}
 }
 
@@ -2543,6 +2539,12 @@ decode_string_and_dup (const char *s)
 	do {
 		if (*s == '\\'){
 			switch (*(++s)){
+			case 's':
+				*p++ = ' ';
+				break;
+			case 't':
+				*p++ = '\t';
+				break;
 			case 'n':
 				*p++ = '\n';
 				break;
@@ -2578,13 +2580,17 @@ escape_string_and_dup (const char *s)
 	q = s;
 	while (*q){
 		len++;
-		if (*q == '\n' || *q == '\\' || *q == '\r' || *q == '\0')
+		if (strchr ("\n\r\t\\", *q) != NULL)
 			len++;
 		q++;
 	}
 	return_value = p = (char *) g_malloc (len + 1);
 	do {
 		switch (*s){
+		case '\t':
+			*p++ = '\\';
+			*p++ = 't';
+			break;
 		case '\n':
 			*p++ = '\\';
 			*p++ = 'n';
@@ -3130,6 +3136,8 @@ ditem_load (ReadBuf *rb,
 						(item->sections, cur_section);
 				}
 				state = IgnoreToEOL;
+			} else if (c == '[') {
+				/* FIXME: probably error out instead of ignoring this */
 			} else {
 				*next++ = c;
 			}
